@@ -1,11 +1,11 @@
 #
-# $Id: Libdnet6.pm 2003 2012-11-16 15:37:48Z gomor $
+# $Id: Libdnet6.pm 2005 2015-01-28 18:59:42Z gomor $
 #
 package Net::Libdnet6;
 use strict;
 use warnings;
 
-our $VERSION = '0.26';
+our $VERSION = '0.27';
 
 use base qw(Exporter);
 
@@ -237,29 +237,53 @@ sub _get_routes_bsd {
    my %devIps;
    for (split('\n', $buf)) {
       my @elts = split(/\s+/);
-      if ($elts[0]) {
-         $elts[0] && $elts[0] =~ s/%[a-z]+[0-9]+//;
-         $elts[1] && $elts[1] =~ s/%[a-z]+[0-9]+//;
-         if (Net::IPv6Addr::is_ipv6($elts[0])) {
-            my $route = {
-               destination => $elts[0],
-               interface   => $elts[-1],
-            };
-            if (Net::IPv6Addr::is_ipv6($elts[1])) {
-               $route->{nextHop} = $elts[1];
-            }
-            push @ifRoutes, $route;
+
+      my $destination = $elts[0] || undef;
+      my $gateway = $elts[1] || undef;
+      my $flags = $elts[2] || undef;
+      my $if = $elts[3] || undef;
+
+      if (defined($destination)) {
+         $destination =~ s/%[a-z]+[0-9]+//;
+      }
+      if (defined($gateway)) {
+         $gateway =~ s/%[a-z]+[0-9]+//;
+      }
+
+      next if ! defined($destination);
+
+      # FreeBSD > 9.x has a new scheme for default routes:
+      # it uses the MAC address of default interface
+      if ($gateway
+      &&  $gateway =~ /^[a-z0-9]{2}:[a-z0-9]{2}:[a-z0-9]{2}:[a-z0-9]{2}:[a-z0-9]{2}:[a-z0-9]{2}$/i) {
+         my $route = {
+            destination => 'default',
+            interface => $if,
+         };
+         if (Net::IPv6Addr::is_ipv6($elts[1])) {
+            $route->{nextHop} = $destination;
          }
-         elsif ($elts[0] eq 'default') {
-            my $route = {
-               destination => $elts[0],
-               interface   => $elts[-1],
-            };
-            if (Net::IPv6Addr::is_ipv6($elts[1])) {
-               $route->{nextHop} = $elts[1];
-            }
-            push @ifRoutes, $route;
+         push @ifRoutes, $route;
+      }
+      elsif (Net::IPv6Addr::is_ipv6($destination)) {
+         my $route = {
+            destination => $destination,
+            interface   => $if,
+         };
+         if (Net::IPv6Addr::is_ipv6($gateway)) {
+            $route->{nextHop} = $gateway;
          }
+         push @ifRoutes, $route;
+      }
+      elsif ($destination eq 'default') {
+         my $route = {
+            destination => $destination,
+            interface => $if,
+         };
+         if (Net::IPv6Addr::is_ipv6($gateway)) {
+            $route->{nextHop} = $gateway;
+         }
+         push @ifRoutes, $route;
       }
    }
 
@@ -451,7 +475,7 @@ Patrice E<lt>GomoRE<gt> Auffret
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2006-2012, Patrice E<lt>GomoRE<gt> Auffret
+Copyright (c) 2006-2015, Patrice E<lt>GomoRE<gt> Auffret
 
 You may distribute this module under the terms of the Artistic license.
 See LICENSE.Artistic file in the source distribution archive.
